#!/usr/bin/env python
#-*- coding:utf-8 -*-
#
#	Copyright (C) Humberto Fraga 2010 <xisberto@gmail.com>
#	
#	This file is part of MagicLibrary.
#	MagicLibrary is another deck manager for Collectible Card Game Magic: The Gathering.
#
#	MagicLibrary is free software: you can redistribute it and/or modify
#	it under the terms of the GNU Affero General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	MagicLibrary is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU Affero General Public License for more details.
#
#	You should have received a copy of the GNU Affero General Public License
#	along with MagicLibrary.  If not, see <http://www.gnu.org/licenses/>.

import magiclibrary_interface
import magiclibrary_gatherer
import os
import threading

class MagicLibrary:
	def __init__(self):
		self.app_version = "0.1rc"
		self.main_dir = os.getcwd()
		self.interface = magiclibrary_interface.Interface(self)
		self.gatherer = magiclibrary_gatherer.Gatherer(self)
	
	def fetchUrl(self, url, complete_callback):
		"""Fethces a url in a thread from GeneratorTask and emits a wait signal in a second GeneratorTask. Executes complete_callback when done.
		"""
		self.waiting = True
		try:
			tasker = threading.Thread(target=self.gatherer.htmlRetrieve, args=(url, self.hookSock, complete_callback))
			tasker.start()
		except IOError:
			self.interface.statusBarUpdate("Error ao conectar com http://gatherer.wizards.com")
		waiter = threading.Thread(target=self.wait)
		waiter.start()
	
	def fetchImg(self, url, image_file, complete_callback):
		self.waiting = True
		tasker = threading.Thread(target=self.gatherer.imgRetrieve, args=(url, image_file, self.hookSock, complete_callback))
		waiter = threading.Thread(target=self.wait)
		tasker.start()
		waiter.start()

	def hookSock(self, count, block_size, total_size):
		progress = count*block_size/float(total_size)
		self.update_progress(progress)
	
	def update_progress(self, progress):
		self.interface.waitProgress(fraction=progress)
		if progress >= 1.0:
			self.waiting = False

	def wait(self):
		"""Waits 0.1 second and yields True until receive stop=True. Then yields False.
		"""
		import time
		while self.waiting:
			time.sleep(0.1)
			self.interface.waitPulse(True)
		self.interface.waitPulse(False)

	def stopWaiter(self, signal):
		"""Stops the waiter, causing interface to exit wait mode.
		"""
		self.waiter.stop()
		self.interface.progressPulse(False)
		self.waiting = False

if __name__ == '__main__':
	app = MagicLibrary()
	app.interface.start()


