#!/usr/bin/env python
#-*- coding:utf-8 -*-
#
#	Copyright (C) Humberto Fraga 2010 <xisberto@gmail.com>
#	
#	This file is part of MagicLibrary.
#	MagicLibrary is another deck manager for Collectible Card Game Magic: The Gathering.
#
#	MagicLibrary is free software: you can redistribute it and/or modify
#	it under the terms of the GNU Affero General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	MagicLibrary is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU Affero General Public License for more details.
#
#	You should have received a copy of the GNU Affero General Public License
#	along with MagicLibrary.  If not, see <http://www.gnu.org/licenses/>.

import pygtk
pygtk.require('2.0')
import gtk
from gtk import gdk
import gobject
import os
import urllib
	
gtk.gdk.threads_init()

class Interface:
	"""This generates the UI and has functions to control it.
	"""
	def __init__(self, controller):
		"""Reads the glade file and sets the main widgets and variables.
		"""
		self.controller = controller

		self.builder = gtk.Builder()
		self.builder.add_from_file("magiclibrary.glade")
		self.builder.connect_signals(self)
		self.window = self.get_widget("window")
		self.statusbar = self.get_widget("statusbar")

		self.window.set_icon_from_file(self.controller.main_dir+"/resources/images/busy/busy.svg")

		self.init_CheckButtons()
		self.init_tv_search()
		self.init_combos()

		#busy variables
		#the actual pass of busy icon
		self.busy_pass = 1
		#the various busy icons
		self.busy_icons = []
		for i in range(0, 10):
			self.busy_icons.append(gtk.gdk.pixbuf_new_from_file(self.controller.main_dir+"/resources/images/busy/busy%s.png" % i))

	def start(self):
		"""Really starts the application.
		"""
		self.window.show()
		self.statusBarUpdate("Application started.")
		gtk.main()
	
	def init_CheckButtons(self):
		for symbol in ("W", "U", "B", "R", "G"):
			self.get_widget("image_%s" % symbol.lower()).set_from_pixbuf(self.getManaPixbuf(symbol))

	def init_tv_search(self):
		tv_search = self.get_widget("treeviewSearch")
		self.search_model = gtk.ListStore(gobject.TYPE_STRING, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING)
		tv_search.set_model(self.search_model)
		column_name = gtk.TreeViewColumn("Card Name")
		column_cost = gtk.TreeViewColumn("Card Cost")
		column_type = gtk.TreeViewColumn("Card Type")
		tv_search.append_column(column_name)
		tv_search.append_column(column_cost)
		tv_search.append_column(column_type)
		cell_name = gtk.CellRendererText()
		column_name.pack_start(cell_name)
		column_name.add_attribute(cell_name, 'text', 0)
		cell_type = gtk.CellRendererText()
		column_type.pack_start(cell_type)
		column_type.add_attribute(cell_type, 'text', 2)
		cell_cost = gtk.CellRendererPixbuf()
		column_cost.pack_start(cell_cost)
		column_cost.add_attribute(cell_cost, 'pixbuf', 1)

	def init_combos(self):
		for combo_name in ["set","format","type"]:
			try:
				list_file = open(self.controller.main_dir+"/resources/lists/%s.list" % combo_name, 'r')
				self.load_combo(combo_name, list_file)
			except:
				pass

	def btnUpdate_activate(self, widget):
		widget.set_sensitive(False)
		self.controller.fetchUrl("http://gatherer.wizards.com", self.updateCombos)

	def updateCombos(self):
		"""One-shot function to update the comboboxes according to each select from search form.
		"""
		self.updateCombo("set")
		self.updateCombo("format")
		self.updateCombo("type")
		self.get_widget("btnUpdate").set_sensitive(True)

	def updateCombo(self, combo_name):
		if combo_name != "format" and combo_name != "set" and combo_name != "type":
			return 1
		list = self.controller.gatherer.get(combo_name)
		list_file = open(self.controller.main_dir+"/resources/lists/%s.list" % combo_name, 'w')
		for line in list:
			list_file.write(line+"\n")
		list_file.close()
		self.load_combo(combo_name, list)

	def load_combo(self, combo_name, list):
		combo = self.get_widget("combo%s" % combo_name.capitalize())
		try:
			model = combo.get_model()
			model.clear()
		except:
			model = gtk.ListStore(str)
			combo.set_model(model)
			cell = gtk.CellRendererText()
			combo.pack_start(cell)
			combo.add_attribute(cell, 'text', 0)
		for item in list:
			model.append([item.strip("\n")])
	
	def updateResults(self):
		card_list = self.controller.gatherer.getCards()
		for card in card_list:
			row = [card["name"]]
			row.append(self.getManaPixbuf(card["mana"]))
			row.append(card["type"])
			row.append(card["multiverseid"])
			self.search_model.append(row)
	
	def deckNew(self, widget):
		self.statusBarUpdate("New deck created.")
	
	def deckOpen(self, widget):
		self.statusBarUpdate("Deck opened.")
	
	def deckSave(self, widget):
		self.statusBarUpdate("Deck saved.")
	
	def deckSaveAs(self, widget):
		self.statusBarUpdate("New deck saved.")
	
	def deckAdd(self, widget):
		self.statusBarUpdate("Card added to deck.")

	def deckDelete(self, widget):
		self.statusBarUpdate("Card deleted from deck.")
	
	def aboutDialogRun(self, widget):
		dialog = self.get_widget("aboutDialog")
		dialog.set_version(self.controller.app_version)
		dialog.run()
		dialog.hide()

	def colors_restrictions(self, widget):
		"""Translated from function UpdateSimpleSearchRestrictions in SearchControls.js in Gatherer page.
		Limits the possibilities of color matching in searches.
		"""
		if widget == self.get_widget("chkColorsMulti"):
			other_check = self.get_widget("chkColorsExclude")
		else:
			other_check = self.get_widget("chkColorsMulti")
		if widget.get_active():
			other_check.set_active(False)
			other_check.set_sensitive(False)
		else:
			other_check.set_sensitive(True)

	def cardSearch(self, widget):
		"""Does the search. The result will be in self.controller.gatherer.getCards.
		Run self.updateResults when done.
		"""
		self.search_model.clear()
		entry = self.get_widget("entrySearch")
		search_string = entry.get_text()
		query = self.buildQuery()
		self.controller.fetchUrl(query, self.updateResults)
		entry.set_text(search_string)
	
	def cardView(self, widget, event):
		(model, iter) = self.get_widget("treeviewSearch").get_selection().get_selected()
		multiverseid = model.get_value(iter, 3)
		image_file = self.controller.main_dir+"/resources/images/cards/%s.jpg" % multiverseid
		if not os.access(image_file, os.F_OK):
			self.controller.fetchImg("http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=%s&type=card" % multiverseid, image_file, self.setCardImage)
		else:
			self.setCardImage(image_file)
	
	def setCardImage(self, image_file):
		self.get_widget("imageCard").set_from_file(image_file)

	def on_entrySearch_icon_press(self, widget, icon, event):
		if icon.value_name == "GTK_ENTRY_ICON_SECONDARY":
			self.cardSearch(widget)

	def appQuit(self, widget):
		gtk.main_quit()
	
	def waitProgress(self, fraction):
		entry = self.get_widget("entrySearch")
		if fraction >= 1.0:
			entry.set_progress_fraction(0.0)
		else:
			entry.set_progress_fraction(fraction)

	def waitPulse(self, pulse):
		"""If pulse>0, the application is busy; update busy icon.
		"""
		entry = self.get_widget("entrySearch")
		if pulse:
			self.statusBarUpdate("Wait while retreving online data.")
			if self.busy_pass == 9:
				self.busy_pass = 0
			else:
				self.busy_pass += 1
			entry.set_icon_from_pixbuf("GTK_ENTRY_ICON_PRIMARY", self.busy_icons[self.busy_pass])
		else:
			self.busy_pass = 0
			entry.set_icon_from_pixbuf("GTK_ENTRY_ICON_PRIMARY", self.busy_icons[0])
			self.statusBarUpdate("")
	
	def statusBarUpdate(self, message):
		context_id = self.statusbar.get_context_id(message)
		self.statusbar.push(context_id, message)

	def get_widget(self, widget_name):
		"""Simplifies getting objects from glade file.
		"""
		return self.builder.get_object(widget_name)
	
	def buildQuery(self):
		"""Builds the query like the search form does.
		"""
		query = "http://gatherer.wizards.com/Pages/Search/Default.aspx?"
		search_string = self.get_widget("entrySearch").get_text()
		if self.get_widget("chkNames").get_active():
			if search_string.find(' ') < 0: #no spaces on search, only one term
				query += "name=+[%s]" % search_string
			else: #multiple terms: "Artifact Land" becomes "+[Artifact]+[Land]"
				subquery = ""
				for term in search_string.split():
					subquery += "+[%s]" % term
				query += "name=%s" % subquery
			if self.get_widget("chkTypes").get_active() or self.get_widget("chkText").get_active():
				query += "||"
		if self.get_widget("chkTypes").get_active():
			if search_string.find(' ') < 0: #no spaces on search, only one term
				query += "type=+[%s]||subtype=+[%s]" % (search_string, search_string)
			else: #multiple terms: "Artifact Land" becomes "+[Artifact]+[Land]"
				subquery = ""
				for term in search_string.split():
					subquery += "+[%s]" % term
				query += "type=%s||subtype=%s" % (subquery, subquery)
			if self.get_widget("chkText").get_active():
				query += "||"
		if self.get_widget("chkText").get_active():
			if search_string.find(' ') < 0: #no spaces on search, only one term
				query += "text=+[%s]" % search_string
			else: #multiple terms: "Artifact Land" becomes "+[Artifact]+[Land]"
				subquery = ""
				for term in search_string.split():
					subquery += "+[%s]" % term
				query += "text=%s" % subquery

		if (self.get_widget("chkW").get_active()) or (self.get_widget("chkU").get_active()) or (self.get_widget("chkB").get_active()) or (self.get_widget("chkR").get_active()) or (self.get_widget("chkG").get_active()):

			if self.get_widget("chkColorsExact").get_active():
				colors_sep = "+"
			else:
				colors_sep = "|"
		
			if self.get_widget("chkColorsExclude").get_active():
				colors_ini = "+@(+"
				colors_end = ")"
			elif self.get_widget("chkColorsMulti").get_active():
				colors_ini = "+^(+"
				colors_end = ")"
			else:
				colors_ini = "+"
				colors_end = ""

			query = query + "&color=" + colors_ini
			for symbol in ("W", "U", "B", "R", "G"):
				if self.get_widget("chk%s" % symbol).get_active():
					query = query + "[%s]" % symbol + colors_sep
			query = query.rstrip(colors_sep)
			if colors_end:
				query = query + colors_end

		for cmb_name in ("format", "set"):
			combo = self.get_widget("combo%s" % cmb_name.capitalize())
			if combo.get_active() > 0:
				iter = combo.get_active_iter()
				query = query + "&%s=" % cmb_name + '["%s"]' % combo.get_model().get_value(iter, 0)

		combo = self.get_widget("comboType")
		if combo.get_active() > 0:
			iter = combo.get_active_iter()
			if query.count('type') > 0:
				query = query.replace("type=", 'type=+[%s]' % combo.get_model().get_value(iter, 0), 1)
			else:
				query = query + '&type=+[%s]' % combo.get_model().get_value(iter, 0)

		return query+"&output=compact"
	
	def getManaPixbuf(self, mana_cost):
		"""Returns a Pixbuf containing a image from resources/images/mana
		"""
		size = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)[0]
		if not mana_cost: return None
		pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, size * len(mana_cost), size)
		for index in range(0, len(mana_cost)):
			symbol = gtk.gdk.pixbuf_new_from_file_at_size(self.controller.main_dir+"/resources/images/mana/%s.png" % mana_cost[index], size, size)
			symbol.copy_area(0, 0, size, size, pixbuf, size * index, 0)
		return pixbuf

#debug code
if __name__ == '__main__':
	interface = Interface(None)
	interface.start()


